// Create closure to keep namespace clean and hide implementation.
(function($) {
  $.fn.applyRange = function(options) {
    var instance = this;

    function getConfig(field, default_value) {
      if (typeof options === 'undefined' ||
        typeof options[field] === 'undefined') {
        return default_value;
      } else {
        // Developer chose to specify form field manually.
        return options[field];
      }
    }

    instance.postcode = getConfig('postcode', instance.find('.postcode'));
    instance.range = getConfig('range', instance.find('.range'));
    instance.message = getConfig('message', instance.find('.message'));
    instance.spinner = getConfig('spinner', instance.find('.spinner'));
    instance.output = getConfig('output', instance.find('.output'));

    instance.distance = getConfig('distance', instance.find('.distance'));
    instance.lat = getConfig('lat', instance.find('.lat'));
    instance.lng = getConfig('lng', instance.find('.lng'));
    instance.nl_fourpp = getConfig('nl_fourpp', instance.find('.nl_fourpp'));

    instance.assemble_func = getConfig('assemble_func', assemble_one);

    instance.postcode.keyup(function() {
      range(instance);
    });
    instance.change(function() {
      range(instance);
    });
  };

  var pro6pp_cache = {};
  function pro6pp_cached_get(obj, url, params, callback) {
    var key = url + $.param(params);
    if (pro6pp_cache.hasOwnProperty(key)) {
      if (typeof callback !== 'undefined') {
        callback(obj, pro6pp_cache[key]);
      }
    } else {
      $.getJSON(url + "?callback=?", params, function(data) {
        pro6pp_cache[key] = data;
        if (typeof callback !== 'undefined') {
          callback(obj, data);
        }
      });
    }
  }

  function range(obj) {
    obj.message.hide().empty();
    var postcode = obj.postcode.val();
    var range = obj.range.val();
    // Trigger on '5408'
    var nl_fourpp_regex = /[0-9]{4,4}\s?/;
    if (nl_fourpp_regex.test(postcode)) {
      var url = 'https://api.pro6pp.nl/v1/range';
      var params = new Object();
      params.auth_key = pro6pp_auth_key;
      params.per_page = 100;
      params.nl_fourpp = postcode;
      // User puts in range in kilometers, API uses meters.
      range = parseInt(range, 10) * 1000;
      params.range = range;
      pro6pp_cached_get(obj, url, params, fillin);
    }
  }

  function assemble_one(result) {
    distance_km = Math.round(result.distance / 1000);
    return "Afstand tot " + result.nl_fourpp + " is " + distance_km + " km<br />";
  }

  function fillin(obj, json) {
    if (json.status == 'ok') {
      obj.output.empty();

      var postcode = obj.postcode.val();
      $.each(json.results, function(i, result) {
        if (result.nl_fourpp === postcode) {
          return true;
        }
        obj.output.append(obj.assemble_func(result)).show();
      });
      if (json.results.length > 1) {
        obj.distance.val(json.results[1].distance);
        obj.nl_fourpp.val(json.results[1].nl_fourpp);
        obj.lat.val(json.results[1].lat);
        obj.lng.val(json.results[1].lng);
      } else {
        translated_message = 'Geen resultaten gevonden';
        obj.message.html(translated_message).show();
      }
    } else {
      var translated_message = json.error.message;
      if (json.error.message == 'nl_fourpp not found') {
        translated_message = 'Onbekende postcode';
      } else if (json.error.message == 'Invalid nl_fourpp format') {
          translated_message = 'Ongeldig postcode formaat';
      } else if (json.error.message == 'invalid target_nl_fourpps format') {
        translated_message = 'Ongeldig target_nl_fourpps formaat';
      }

      obj.message.html(translated_message).show();
    }
  }
})(jQuery);
